#!/bin/bash
set -e

# Source environment variables
source /app/.env

# Function to generate a random password
generate_password() {
    cat /dev/urandom | tr -dc 'a-zA-Z0-9' | fold -w 16 | head -n 1
}

# Create a secrets directory in data (which is writable)
mkdir -p /app/data/secrets

# Check for initialization status
if [[ ! -f /app/data/.initialized ]]; then
    echo "Fresh installation, initializing..."
    
    # Generate and store passwords
    if [ -z "$ELASTIC_PASSWORD" ]; then
        ELASTIC_PASSWORD=$(generate_password)
        echo "Generated new secure password for Elasticsearch user 'elastic'"
    fi
    echo "$ELASTIC_PASSWORD" > /app/data/secrets/elastic_password
    
    if [ -z "$KIBANA_PASSWORD" ]; then
        KIBANA_PASSWORD=$(generate_password)
        echo "Generated new secure password for Elasticsearch user 'kibana_system'"
    fi
    echo "$KIBANA_PASSWORD" > /app/data/secrets/kibana_password
    
    # Mark as initialized
    touch /app/data/.initialized
    echo "Initialization complete."
else
    echo "Loading existing configuration..."
    # Load existing passwords
    if [ -f "/app/data/secrets/elastic_password" ]; then
        ELASTIC_PASSWORD=$(cat /app/data/secrets/elastic_password)
    fi
    
    if [ -f "/app/data/secrets/kibana_password" ]; then
        KIBANA_PASSWORD=$(cat /app/data/secrets/kibana_password)
    fi
fi

# Set default variables
STACK_VERSION=${STACK_VERSION:-8.17.3}
CLUSTER_NAME=${CLUSTER_NAME:-cloudron-cluster}
LICENSE=${LICENSE:-basic}

# Ensure data directories exist
mkdir -p /app/data/elasticsearch
mkdir -p /app/data/certs

# Set permissions
chmod -R 777 /app/data

# Check if certificates exist, if not create them
if [ ! -f /app/data/certs/ca.crt ]; then
    echo "Creating certificates..."
    # Create temporary container to generate certificates
    docker run --rm \
        -v /app/data/certs:/usr/share/elasticsearch/config/certs \
        --name es_certs \
        docker.elastic.co/elasticsearch/elasticsearch:${STACK_VERSION} \
        /bin/bash -c "
            elasticsearch-certutil ca --silent --pem -out config/certs/ca.zip &&
            unzip config/certs/ca.zip -d config/certs &&
            elasticsearch-certutil cert --silent --pem -out config/certs/certs.zip --ca-cert config/certs/ca/ca.crt --ca-key config/certs/ca/ca.key --name es01 --dns es01,localhost --ip 127.0.0.1 &&
            unzip config/certs/certs.zip -d config/certs
        "
    echo "Certificates created."
fi

# Print the network interfaces for debugging
echo "Network interfaces:"
ip addr show

# Start Elasticsearch
echo "Starting Elasticsearch..."
docker run \
    -d \
    --restart=always \
    --name elasticsearch \
    -v /app/data/elasticsearch:/usr/share/elasticsearch/data \
    -v /app/data/certs:/usr/share/elasticsearch/config/certs \
    -e node.name=es01 \
    -e cluster.name=${CLUSTER_NAME} \
    -e discovery.type=single-node \
    -e ELASTIC_PASSWORD=${ELASTIC_PASSWORD} \
    -e bootstrap.memory_lock=true \
    -e xpack.security.enabled=true \
    -e xpack.security.http.ssl.enabled=false \
    -e xpack.license.self_generated.type=${LICENSE} \
    -e xpack.ml.use_auto_machine_memory_percent=true \
    -e "ES_JAVA_OPTS=-Xms512m -Xmx512m" \
    -e network.host=0.0.0.0 \
    --ulimit memlock=-1:-1 \
    -p 0.0.0.0:9200:9200 \
    -p 0.0.0.0:9300:9300 \
    docker.elastic.co/elasticsearch/elasticsearch:${STACK_VERSION}

# Wait for Elasticsearch to be up - try both localhost and 0.0.0.0
echo "Waiting for Elasticsearch to start..."
until $(curl --output /dev/null --silent --head --fail http://localhost:9200 || curl --output /dev/null --silent --head --fail http://0.0.0.0:9200); do
    printf '.'
    sleep 5
done
echo "Elasticsearch is up and running!"

# If kibana password is set, configure the kibana_system user
if [ ! -z "$KIBANA_PASSWORD" ]; then
    echo "Setting kibana_system user password..."
    until curl -s -X POST -u "elastic:${ELASTIC_PASSWORD}" -H "Content-Type: application/json" http://localhost:9200/_security/user/kibana_system/_password -d "{\"password\":\"${KIBANA_PASSWORD}\"}" | grep -q "^{}"; do
        echo "Waiting for Elasticsearch to be ready to set kibana_system password..."
        sleep 5
    done
    echo "kibana_system password set."
fi

# Display the credentials
echo "-----------------------------"
echo "Elasticsearch is ready to use!"
echo "URL: http://localhost:9200"
echo ""
echo "Authentication credentials:"
echo "  User: elastic"
echo "  Password: $ELASTIC_PASSWORD"
if [ ! -z "$KIBANA_PASSWORD" ]; then
    echo ""
    echo "Kibana system credentials:"
    echo "  User: kibana_system"
    echo "  Password: $KIBANA_PASSWORD"
fi
echo "-----------------------------"

# Create a credentials file for reference
cat > /app/data/credentials.txt << EOL
Elasticsearch credentials:
URL: http://localhost:9200
User: elastic
Password: $ELASTIC_PASSWORD

Kibana system credentials:
User: kibana_system
Password: $KIBANA_PASSWORD
EOL

echo "Credentials saved to /app/data/credentials.txt"

# Keep script running
exec tail -f /dev/null 